﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using UnityEngine;

internal static class RaftVRMSettings
{
    private static Dictionary<string, string[]> settingsDic = new Dictionary<string, string[]>();
    private static Dictionary<string, bool> realtimeDic = new Dictionary<string, bool>();
    public static Dictionary<(string playerName, string key), object> cacheDic = new Dictionary<(string playerName, string key), object>();

    private static string GetSettingsPath(string playerName)
    {
        return PathUtil.Combine(RaftVRM.RaftVRMPath, "settings_" + playerName + ".txt");
    }

    private static string[] ReadSettingsFile(string playerName)
    {
        var path = GetSettingsPath(playerName);
        if (!File.Exists(path))
        {
            Debug.LogWarning($"[RaftVRM] Settings file (settings_{playerName}.txt) does not exist.");
            return null;
        }

        return File.ReadAllLines(path);
    }

    public static void UpdateSettingsData(string playerName)
    {
        settingsDic[playerName] = ReadSettingsFile(playerName);
    }

    public static bool Init(string playerName)
    {
        if (settingsDic.ContainsKey(playerName)) return false;

        realtimeDic[playerName] = false;

        var data = ReadSettingsFile(playerName);
        if (data == null)
        {
            Debug.LogWarning($"[RaftVRM] Settings file (settings_{playerName}.txt) does not exist.");
            return false;
        }

        settingsDic[playerName] = data;
        realtimeDic[playerName] = ReadBool(playerName, "RealtimeSettings", false);

        return realtimeDic[playerName];
    }

    private static string ReadSettings(string playerName, string key)
    {
        if (!settingsDic.TryGetValue(playerName, out var data))
        {
            data = ReadSettingsFile(playerName);
            if (data != null)
            {
                settingsDic[playerName] = data;
            }
            else
            {
                return null;
            }
        }

        string res = null;
        foreach (var line in data)
        {
            var l = line.Replace(" ", "");
            if (l.IndexOf(key + "=") == 0)
            {
                res = l.Replace(key + "=", "");
            }
        }

        return res;
    }

    public static int ReadInt(string playerName, string key, int defaultValue = 0)
    {
        if (!realtimeDic[playerName] && cacheDic.TryGetValue((playerName, key), out var cache))
        {
            return (int)cache;
        }
        var str = ReadSettings(playerName, key);
        if (int.TryParse(str, NumberStyles.Integer, CultureInfo.InvariantCulture, out var res))
        {
            cacheDic[(playerName, key)] = res;
            return res;
        }
        return defaultValue;
    }

    public static float ReadFloat(string playerName, string key, float defaultValue = 0.0f)
    {
        if (!realtimeDic[playerName] && cacheDic.TryGetValue((playerName, key), out var cache))
        {
            return (float)cache;
        }
        var str = ReadSettings(playerName, key);
        if (float.TryParse(str, NumberStyles.Float, CultureInfo.InvariantCulture, out var res))
        {
            cacheDic[(playerName, key)] = res;
            return res;
        }
        return defaultValue;
    }

    public static bool ReadBool(string playerName, string key, bool defaultValue = false)
    {
        if (!realtimeDic[playerName] && cacheDic.TryGetValue((playerName, key), out var cache))
        {
            return (bool)cache;
        }
        var str = ReadSettings(playerName, key);
        if (bool.TryParse(str, out var res))
        {
            cacheDic[(playerName, key)] = res;
            return res;
        }
        return defaultValue;
    }

    public static Vector3 ReadVector3(string playerName, string key, Vector3 defaultValue = default)
    {
        if (!realtimeDic[playerName] && cacheDic.TryGetValue((playerName, key), out var cache))
        {
            return (Vector3)cache;
        }
        var str = ReadSettings(playerName, key);
        if (str == null) return defaultValue;

        var match = new Regex("\\((?<x>[^,]*?),(?<y>[^,]*?),(?<z>[^,]*?)\\)").Match(str);
        if (match.Success == false) return defaultValue;
        try
        {
            var res = new Vector3()
            {
                x = float.Parse(match.Groups["x"].Value),
                y = float.Parse(match.Groups["y"].Value),
                z = float.Parse(match.Groups["z"].Value)
            };
            cacheDic[(playerName, key)] = res;
            return res;
        }
        catch
        {
            return defaultValue;
        }
    }
}